{$I DFS.INC}  { Standard defines for all Delphi Free Stuff components }

{-----------------------------------------------------------------------------}
{ TjmrTaskBarInfo v1.11                                                       }
{-----------------------------------------------------------------------------}
{ A component that provides information about the Windows 95 Task Bar.        }
{ May also be used to provide information about other Win32 Application Bars. }
{                                                                             }
{ Written by Jon M. Robertson.                                                }
{ Copyright 1998-1999, Computer Solutions.  All Rights Reserved.              }
{                                                                             }
{ This component can be freely used and distributed in commercial and private }
{ environments, provided this notice is not modified in any way and there is  }
{ no charge for it other than nomial handling fees.  Contact me directly for  }
{ modifications to this agreement.                                            }
{                                                                             }
{ DFS.INC is Copyright 1996, Brad Stowers.  All Rights Reserved.  DFS.INC     }
{ is included courtsey of Brad.  He may be contacted at bstowers@pobox.com    }
{-----------------------------------------------------------------------------}
{ Feel free to contact me if you have any questions, comments or suggestions  }
{ at JonRobertson@pobox.com                                                   }
{ The lateset version will always be available on the web at:                 }
{   http://www.delphifreestuff.com/ in the Other's Stuff section              }
{ See TaskBarInfo.txt for notes, known issues, and revision history.          }
{-----------------------------------------------------------------------------}
{ Date last modified:  July 28th, 1999                                        }
{-----------------------------------------------------------------------------}
{ This format was borrowed from Brad Stowers.  Primarily because he didn't    }
{ mind and I didn't see anything wrong with it.  Also, since he is kind       }
{ enough to provide a home for this component, I thought the DFS format       }
{ would be more familar to him and other visitors.  -- Jon                    }
{-----------------------------------------------------------------------------}

// C++ Builder 3 requires ObjExportAll if you want to use run-time packages.

{$ifdef DFS_CPPB_3_UP}
  {$ObjExportAll On}
{$endif}

// When the component is installed in a package, the DCR file is automatically
// included in the package.  However, compilers which do not support packages
// do not automatically include the DCR in the library.

{$ifndef DFS_COMPILER_3_UP}
  {$R TaskBarInfo.dcr}
{$endif}

// Turn on RTTI.  Since TBarRect is used as a property of TaskBarInfo, it needs
// RTTI.  Originally, deriving TBarRect from TPersistent took care of this.
// However, this way saves a couple of KiloBytes.

{$M+}

unit TaskBarInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ShellAPI, DsgnIntf;

const
  jmrTASKBARNAME         = 'Shell_TrayWnd';
  jmrTASKBARINFO_VERSION = '1.10';

type
  TjmrBarPosition = (abeBottom, abeLeft, abeRight, abeTop);

  TjmrBarRect = class
  public
    FRect: TRect;
    function  GetRect: TRect;
    procedure SetRect(R: TRect);
  published
    property Left:   Integer read FRect.Left   write FRect.Left   stored false;
    property Right:  Integer read FRect.right  write FRect.right  stored false;
    property Top:    Integer read FRect.top    write FRect.top    stored false;
    property Bottom: Integer read FRect.bottom write FRect.bottom stored false;
  end;

  TjmrTaskBarInfo = class(TComponent)
  protected
    FWindowName:        String;
    FWindowHandle:      HWnd;
    FBarFound:          Boolean;
    FBarRect:           TjmrBarRect;
    FBarHeight:         Integer;
    FBarWidth:          Integer;

{$ifdef WIN32}
    FAutoHide:          Boolean;
    FAlwaysOnTop:       Boolean;
    FBarHidden:         Boolean;
    FBarPosition:       TjmrBarPosition;

    FMonitorWindow:     HWND;
    FOnBarChange:       TNotifyEvent;
{$endif}

    procedure GetInfo;

    procedure SetWindowName(value: String);
    procedure SetInt(value: Integer);
    procedure SetBool(value: Boolean);

{$ifdef WIN32}
    procedure SetBarPos(value: TjmrBarPosition);
    procedure DoOnBarChange;
    procedure WndProc(var Msg: TMessage);
{$endif}

  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;

  published
    property WindowName:  string       read FWindowName  write SetWindowName;
    property BarRect:     TjmrBarRect     read FBarRect     write FBarRect  stored false;
    property BarHeight:   integer      read FBarHeight   write SetInt    stored false;
    property BarWidth:    integer      read FBarWidth    write SetInt    stored false;
    property BarFound:    boolean      read FBarFound    write SetBool   stored false;
    property BarHidden:   boolean      read FBarHidden   write SetBool   stored false;

{$ifdef WIN32}
    property AutoHide:    boolean      read FAutoHide    write SetBool   stored false;
    property AlwaysOnTop: boolean      read FAlwaysOnTop write SetBool   stored false;
    property BarPosition: TjmrBarPosition read FBarPosition write SetBarPos stored false;
    property OnBarChange: TNotifyEvent read FOnBarChange write FOnBarChange;
{$endif}

  end;

// A component editor (not really) to display the version of the component.

  TjmrTaskBarInfoEditor = class(TDefaultEditor)
  public
    function GetVerb(Index : Integer): string; override;
    function GetVerbCount : Integer; override;
  end;

procedure Register;

implementation

{$ifdef WIN32}
const
  jmrABM_MONITOR_MESSAGE = WM_USER + 112;
{$endif}

// TjmrBarRect methods to simplify use with TRects.

function TjmrBarRect.GetRect: TRect;
begin
  Result := FRect;
end;

procedure TjmrBarRect.SetRect(R: TRect);
begin
  FRect := R;
end;

// Function to modify the style of a Window.

{$ifdef WIN32}
function ModifyStyle(hWnd: THandle; nStyleOffset: Integer; dwRemove: DWORD;
   dwAdd: DWORD; nFlags: UINT) : boolean;
var
  dwStyle : DWORD;
  dwNewStyle : DWORD;
begin
  dwStyle := GetWindowLong(hWnd, nStyleOffset);
  dwNewStyle := (dwStyle and (not dwRemove)) or dwAdd;

  if dwStyle = dwNewStyle then begin
    Result := False;
    Exit;
  end;

  SetWindowLong(hWnd, nStyleOffset, dwNewStyle);

  if nFlags <> 0 then
    SetWindowPos(hWnd, 0, 0, 0, 0, 0,
      SWP_NOSIZE or SWP_NOMOVE or SWP_NOZORDER or SWP_NOACTIVATE or nFlags);

  Result := True;
end;
{$endif}

// In a Win32 environment, create our own app bar so we can see the messages
// generated by the Task Bar.

constructor TjmrTaskBarInfo.Create(AOwner: TComponent);

{$ifdef WIN32}
var
  ABD: TAppBarData;
{$endif}

begin

  inherited Create(AOwner);

  // Create the FBarRect object.

  BarRect := TjmrBarRect.Create;

  // Set the window name here because Strings are not allowed for default values.

  WindowName := jmrTASKBARNAME;

{$ifdef WIN32}
  // Create our own appbar window so the system will tell us about changes to
  // the taskbar.  Non-autohide app bars tell each other about what they are
  // up to through ABN_xxx wparams sent to the appbar's wndproc

  FMonitorWindow := AllocateHWnd(WndProc);

  // Modifying the style probabyl really isn't needed since this will never
  // be visible, but just to be safe I've done it anyway.

  ModifyStyle(FMonitorWindow, GWL_STYLE, WS_SYSMENU or WS_MAXIMIZEBOX or
     WS_MINIMIZEBOX, 0, SWP_HIDEWINDOW);

  ModifyStyle(FMonitorWindow, GWL_EXSTYLE, WS_EX_APPWINDOW, WS_EX_TOOLWINDOW, 0);

  // Setup the AppBarInfo structure and register our app bar with Win32.

  FillChar(ABD, SizeOf(TAppBarData), #0);
  ABD.cbSize := SizeOf(TAppBarData);
  ABD.hWnd := FMonitorWindow;
  ABD.uCallbackMessage := jmrABM_MONITOR_MESSAGE;
  SHAppBarMessage(ABM_NEW, ABD);
{$endif}

  // Get the information about the task bar.

  GetInfo;

end;

// In a Win32 environment, remove our app bar and destroy it.

destructor TjmrTaskBarInfo.Destroy;

{$ifdef WIN32}
var
  ABD: TAppBarData;
{$endif}

begin

{$ifdef WIN32}
  FillChar(ABD, SizeOf(TAppBarData), #0);
  ABD.cbSize := SizeOf(TAppBarData);
  ABD.hWnd := FMonitorWindow;
  SHAppBarMessage(ABM_REMOVE, ABD);
  DeallocateHWnd(FMonitorWindow);
{$endif}

  FBarRect.Free;

  inherited Destroy;

end;

// Get the information about the task bar.

procedure TjmrTaskBarInfo.GetInfo;
var

{$ifdef WIN32}
  abState: UINT;
  barData: TAppBarData;
{$else}
  R:       TRect;
{$endif}

begin

  // If the task bar was not found, then clear the bar information and exit the
  // procedure.

  if FWindowHandle = 0 then begin
    FBarFound       := false;
    FBarRect.Top    := 0;
    FBarRect.Bottom := 0;
    FBarRect.Left   := 0;
    FBarRect.Right  := 0;
    FBarHeight      := 0;
    FBarWidth       := 0;
    exit;
  end;

  // Since H <> 0, the task bar was found.

  FBarFound := true;

  // For Win16, determine the coordinates of the task bar's window.  That's all
  // we can do without calling Win32 routines.

{$ifndef WIN32}

  GetWindowRect(FWindowHandle, R);

  FBarRect.SetRect(R);
  FBarHeight      := FBarRect.Bottom - FBarRect.Top;
  FBarWidth       := FBarRect.Right  - FBarRect.Left;

{$else}

  barData.cbSize := SizeOf(barData);
  barData.hWnd   := FWindowHandle;

  SHAppBarMessage(ABM_GETTASKBARPOS, barData);

  FBarRect.SetRect(barData.rc);
  FBarHeight      := FBarRect.Bottom - FBarRect.Top;
  FBarWidth       := FBarRect.Right  - FBarRect.Left;

  abState := SHAppBarMessage(ABM_GETSTATE, barData);

  if (abState and ABS_ALWAYSONTOP) > 0 then FAlwaysOnTop := True
  else                                      FAlwaysOnTop := False;

  if (abState and ABS_AUTOHIDE) > 0    then FAutoHide := True
  else                                      FAutoHide := False;

  if (FBarHeight = 4) or (FBarWidth = 4) then FBarHidden := true
  else                                        FBarHidden := false;

{$endif}

  if (FBarRect.Top > 0) then begin
    FBarPosition := abeBottom;
  end
  else if (FBarRect.Left > 0) then begin
    FBarPosition := abeRight;
  end
  else if (FBarRect.Right > Screen.Width) then begin
    FBarPosition := abeTop;
  end
  else begin
    FBarPosition := abeLeft;
  end;

end;

// Procedure to set properties.

procedure TjmrTaskBarInfo.SetInt(Value: Integer);
begin
end;

procedure TjmrTaskBarInfo.SetBool(Value: Boolean);
begin
end;

{$ifdef WIN32}
procedure TjmrTaskBarInfo.SetBarPos(value: TjmrBarPosition);
begin
end;
{$endif}

procedure TjmrTaskBarInfo.SetWindowName(value: String);
begin

  // If the user didn't specify a name, then set the default name.

  if value = '' then FWindowName := jmrTASKBARNAME
  else               FWindowName := value;

  // Locate the task bar.

  FWindowHandle := FindWindow(pchar(FWindowName), nil);

end;

// Prcedure triggered by Windows messages.

procedure TjmrTaskBarInfo.DoOnBarChange;
begin
  GetInfo;

  if assigned(FOnBarChange) then FOnBarChange(Self);
end;

// In Win32, this procedure receives ABN_xxx messages from other application
// bars.  I would like to use this to trigger the ON_BAR_CHANGE event.

{$ifdef WIN32}
procedure TjmrTaskBarInfo.WndProc(var Msg: TMessage);
begin
  with Msg do begin

    if Msg = jmrABM_MONITOR_MESSAGE then begin

      // You can check WParam here for the ABN_xxx messages to find out what
      // just happened.

      case WParam of
        ABN_STATECHANGE:   DoOnBarChange;
        ABN_POSCHANGED:    DoOnBarChange;
//        ABN_FULLSCREENAPP:
//        ABN_WINDOWARRANGE:
      end;

      Result := 1;
    end else
      Result := DefWindowProc(FMonitorWindow, Msg, wParam, lParam);
  end;
end;
{$endif}

// Component Editor that displays version of component.

function TjmrTaskBarInfoEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TjmrTaskBarInfoEditor.GetVerb(Index: Integer): AnsiString;
begin
  Result := 'Version ' + jmrTASKBARINFO_VERSION;
end;

// Procedure which registers the component on the user's palette.

procedure Register;
begin
{$ifdef DFS_COMPILER_3_UP}
  RegisterComponents('Win32', [TjmrTaskBarInfo]);
{$else}
  RegisterComponents('Win95', [TjmrTaskBarInfo]);
{$endif}
  RegisterComponentEditor(TjmrTaskBarInfo, TjmrTaskBarInfoEditor);
end;

end.
